#!/usr/bin/env python
'''Main frontend executable'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"
__version__ = "0.1"

import os
import sys
import gobject
import ConfigParser
from ConfigParser import ParsingError
from frontend.frontend_client import FrontendClient

from utils.configuration import Configuration

from backend.core.message import Message
from backend.core.message_type_priority import MessageType, MessagePriority
from backend.core.message_bus_proxy import MessageBusProxy
from utils.system_tray_icon import SystemTrayIcon

try:
    import pygtk
    pygtk.require("2.4")
except:
    pass
try:
    import gtk
except:
    sys.exit(1)

#FIXME: We really could do with a shorter way to get configuration options

CONFIG_FILE = os.path.expanduser(
    Configuration().get_cfg_path() + 'preferences.conf')

config = ConfigParser.ConfigParser()

try:
    config.readfp(open(CONFIG_FILE))
except ParsingError:
    print "Syntax error in configuration file: " + CONFIG_FILE
    print "Execution aborted!"
    sys.exit(1)
except IOError:
    print "Configuration file is missing: " + CONFIG_FILE
    print "Execution aborted!"
    #FIXME: We should create a default preferences file if file is missing
    sys.exit(1)
except:
    print "Unknown error occured when preferences file was loaded!"
    print "Execution aborted!"
    sys.exit(1)

start_icon = config.getboolean("General", "display_icon")
notify = config.getboolean("General", "display_notifications")

del(config)

#If tray icon is to be displayed, start the reeeeeaaaaallly lengthy process
#needed to start it up.
#FIXME: Can somebody look into this code and see if you can shorten it?
if start_icon:
    message_dict = { MessageType.RECORDING_STARTED : MessagePriority.VERY_LOW,
                 MessageType.RECORDING_STOPPED : MessagePriority.VERY_LOW}

    gobject.threads_init()
    gtk.gdk.threads_init()

    # Create tray icon
    gtk.gdk.threads_enter()

    #Start SystemTrayIcon with show on start True
    tray_icon = SystemTrayIcon(start_frontend=True, notify=notify)
    gtk.gdk.threads_leave()

    try:
        # Open connection to backend
        proxy = MessageBusProxy(message_dict, tray_icon.get_message_handler(),
            "system-tray-icon")
        proxy.connectToMessageBus()
        proxy.start()
        tray_icon.get_message_handler().set_message_bus_proxy(proxy)

    except:
        print "Backend is not running. Start backend and try again."
        sys.exit(1)

    gtk.gdk.threads_enter()
    gtk.main()
    gtk.gdk.threads_leave()
else:
    # Start frontend
    FrontendClient()
