'''QuestionDialog - Screen displays a question label and buttons YES and NO.'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import clutter

from frontend.user_event import UserEvent
from frontend.gui.screens.screen import Screen
from frontend.gui.transitions.transition import Transition

from frontend.gui.widgets.text_menu import TextMenu
from frontend.gui.widgets.justified_text_menu_item import JustifiedTextMenuItem

class QuestionDialog(Screen):
    """
    QuestionDialog
    
    Dialog screen that contains question label and two buttons: yes and no.
    This is used when we want confirmation from the user for some operation.
    """
    
    def __init__(self, ui, frontend, data):
        """
        Initialize screen
        
        @param frontend: FrontendClient object
        @param theme: Theme object
        @param data: Userdata [0]=screen, [1]=callback, [2]=question string
        """
        Screen.__init__(self, ui.STAGE_WIDTH, ui.STAGE_HEIGHT)

        self.ui = ui
        self.frontend = frontend
        self.theme = frontend.getTheme()
        self.current_screen = data[0]
        self.callback = data[1]
        self.group = clutter.Group()
        
        #FIXME: Replace rect with beautiful cairo graphics
        rect = clutter.Rectangle()
        rect.set_color((0, 0, 0, 233))
        rect.set_size(740, 200)
        rect.set_position(280, 280)
        rect.show()
        self.group.add(rect)
        
        question = clutter.Label()
        question.set_text(data[2])
        question.set_font_name(self.theme.getFont() + ' 32px')
        question.set_color(self.theme.getColor("title"))
        question.set_position(300, 300)
        question.set_width(700)
        question.show()
        self.group.add(question)
        
        self.buttons = TextMenu(self.theme,
            self.frontend.get_configuration().show_effects())
        self.buttons.set_orientation(self.buttons.HORIZONTAL)
        self.buttons.set_item_size(350, 60)
        self.buttons.set_visible_column_count(2)
        
        yes_button = JustifiedTextMenuItem(350, 60, self.theme, "Yes",
            JustifiedTextMenuItem.CENTER)
        yes_button.set_userdata("yes")
        self.buttons.add_actor(yes_button)
        
        no_button = JustifiedTextMenuItem(350, 60, self.theme, "No",
            JustifiedTextMenuItem.CENTER)
        no_button.set_userdata("no")
        self.buttons.add_actor(no_button)
        
        self.buttons.set_position(300, 380)
        self.buttons.set_active(True)
        self.buttons.show()
        self.group.add(self.buttons)
        
        self.group.show()
        ui.getStage().add(self.group)
    
    def get_group(self):
        """
        Get all actors of the screen.
        @return clutter.Group 
        """
        return self.group

    def get_type(self):
        """Return screen type."""
        return Screen.DIALOG
    
    def get_name(self):
        """Return screen name (human readble)"""
        return "QuestionDialog"
    
    def handle_user_event(self, event):   
        """ 
        Handle screen specific user events
        @param event: UserEvent object
        """
        event_type = event.get_type()
        
        if event_type == UserEvent.NAVIGATE_LEFT:
            self.buttons.move(self.buttons.LEFT)
        
        elif event_type == UserEvent.NAVIGATE_RIGHT:
            self.buttons.move(self.buttons.RIGHT)
            
        elif event_type == UserEvent.NAVIGATE_SELECT:
            if  self.buttons.get_current_menuitem().get_userdata() == "yes":
                self.callback()
                # This is not always possible
                self.ui.changeScreen(self.current_screen, Transition.BACKWARD)
            else:
                self.ui.changeScreen(self.current_screen, Transition.BACKWARD)


