'''VideoScreen - Screen allows user to browse video library content'''

__licence__ = "GPLv2"
__copyright__ = "2008, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import clutter
from frontend.gui.screens.screen import Screen

# Widgets
from frontend.gui.widgets.tab_group import TabGroup
from frontend.gui.tabs.movies_tab import MoviesTab
from frontend.gui.tabs.series_tab import SeriesTab
from frontend.gui.tabs.video_clips_tab import VideoClipsTab

class VideoScreen(Screen):
    """
    VideoScreen
    
    This screen contains tabs for different video types. This is the top level
    UI for video library.
    """
    
    def __init__(self, ui, frontend):
        """
        Initialize screen
        
        @param frontend: FrontendClient object
        @param theme: Theme object
        """
        self.group = clutter.Group()
        Screen.__init__(self, ui.STAGE_WIDTH, ui.STAGE_HEIGHT)
        
        self.ui = ui
        self.frontend = frontend
        self.theme = frontend.getTheme()
        self.library = self.frontend.get_video_library()
        
        # Screen Title (Displayed at the bottom left corner)
        screen_title = clutter.Label()
        screen_title.set_font_name(self.theme.getFont() + " " + \
            str(self.ui.y(0.13)) + "px")
        screen_title.set_color(self.theme.getColor("screentitle"))
        screen_title.set_text("Videos")
        screen_title.set_name("screen_title")
        screen_title.set_position(self.ui.x(0), self.ui.y(0.87))
        screen_title.show()
        self.group.add(screen_title)
        
        # Tabs
        self.tab_group = TabGroup(0.95, 0.13, self.theme.getColor("title"))
        tab1 = MoviesTab(self.frontend, "Movies")
        tab2 = SeriesTab(self.frontend, "TV-Series")
        #tab3 = MusicVideosTab(self.frontend, "Music videos")
        tab4 = VideoClipsTab(self.frontend, "Video clips")
        self.tab_group.add_tab(tab1)
        self.tab_group.add_tab(tab2)
        #self.tab_group.add_tab(tab3)
        self.tab_group.add_tab(tab4)
        self.tab_group.set_y(self.ui.y(0.1))
        self.tab_group.set_anchor_point_from_gravity(clutter.GRAVITY_CENTER)
        self.tab_group.set_x(self.ui.x(0.5))
        self.tab_group.show()
        self.tab_group.set_active(True)
        self.group.add(self.tab_group)

        self.group.add(tab1)
        self.group.add(tab2)
        #self.group.add(tab3)
        self.group.add(tab4)
        
        self.group.show()
        ui.getStage().add(self.group)
    
    def get_group(self):
        """
        Get all actors of the screen.
        @return clutter.Group 
        """
        return self.group

    def get_type(self):
        """
        Get screen type.
        @return: Integer (Constant variable from Screen class)
        """
        return Screen.NORMAL
    
    def get_name(self):
        """
        Get screen name (human readable)
        @return: String
        """
        return "VideoScreen"
    
    def handle_user_event(self, event):   
        """ 
        Handle screen specific user events
        @param event: UserEvent object
        """
        self.tab_group.handle_user_event(event)


