'''WeatherScreen - Screen allows user to view current weather conditions'''

__licence__ = "GPLv2"
__copyright__ = "2008, Jamie Bennett"
__author__ = "Jamie Bennett <jamie@linuxuk.org>"

import clutter
import gtk

from frontend.gui.screens.screen import Screen
from utils.weather import Weather

# Configuration
from utils.configuration import Configuration

class WeatherScreen(Screen):
    """
    WeatherScreen

    Front end weather forecast screen
    """

    def __init__(self, ui, frontend):
        """
        Initialize screen

        @param frontend: FrontendClient object
        @param theme: Theme object
        """
        loc = Configuration().get_weather_location()
        metric = Configuration().get_weather_metric()
        self.weather = Weather(loc, metric)
        self.weather.refresh()
        self.group = clutter.Group()
        Screen.__init__(self, ui.STAGE_WIDTH, ui.STAGE_HEIGHT)

        self.ui = ui
        self.frontend = frontend
        self.theme = frontend.getTheme()

        # Screen Title (Displayed at the bottom left corner)
        screen_title = clutter.Label()
        screen_title.set_font_name(self.theme.getFont() + " " + \
            str(self.ui.y(0.13)) + "px")
        screen_title.set_color(self.theme.getColor("screentitle"))
        screen_title.set_text("Weather")
        screen_title.set_name("screen_title")
        screen_title.set_position(self.ui.x(0), self.ui.y(0.87))
        screen_title.show()
        self.group.add(screen_title)
  
        # Show the location
        location = clutter.Label()
        location.set_font_name(self.theme.getFont() + " bold " + \
            str(self.ui.y(0.04167)) + "px")
        location.set_color(self.theme.getColor("text"))
        if (self.weather.get_location() == ''):
            location.set_text("No weather location defined!")
        else:
            location.set_text(self.weather.get_location())

        location.set_position(self.ui.x(0.40), self.ui.y(0.13))
        location.show()
        self.group.add(location)

        forecasts = self.weather.get_forecasts()
        day = forecasts[0]

        # Weather Image
        weather_image = clutter.Texture()
        pix_buffer = gtk.gdk.pixbuf_new_from_file(day["Image"])
        weather_image.set_position(self.ui.x(0.10), self.ui.y(0.15))
        weather_image.set_pixbuf(pix_buffer)
        weather_image.show()
        self.group.add(weather_image)
  
        # Show todays weather
        day1 = clutter.Label()
        day1.set_font_name(self.theme.getFont() + " bold " + \
            str(self.ui.y(0.04167)) + "px")
        day1.set_color(self.theme.getColor("text"))
        day1.set_text(day["Day"])
        day1.set_position(self.ui.x(0.13), self.ui.y(0.35))
        day1.show()
        self.group.add(day1)

        # Show todays high, low and condition
        day1_more = clutter.Label()
        day1_more.set_font_name(self.theme.getFont() + \
            str(self.ui.y(0.04167)) + "px")
        day1_more.set_color(self.theme.getColor("text"))
        day1_more.set_text("High: " + day["High"] + "   Low: " + \
            day["Low"] + "\nCondition: " + day["Condition"])
        day1_more.set_position(self.ui.x(0.13), self.ui.y(0.40))
        day1_more.show()
        self.group.add(day1_more)

        # Show tomorrows weather
        day = forecasts[1]

        # Weather Image
        weather_image = clutter.Texture()
        pix_buffer = gtk.gdk.pixbuf_new_from_file(day["Image"])
        weather_image.set_position(self.ui.x(0.10), self.ui.y(0.50))
        weather_image.set_pixbuf(pix_buffer)
        weather_image.show()
        self.group.add(weather_image)

        # Show tomorrows weather
        day2 = clutter.Label()
        day2.set_font_name(self.theme.getFont() + " bold " + \
            str(self.ui.y(0.04167)) + "px")
        day2.set_color(self.theme.getColor("text"))
        day2.set_text(day["Day"])
        day2.set_position(self.ui.x(0.13), self.ui.y(0.70))
        day2.show()
        self.group.add(day2)

        # Show tomorrows high, low and condition
        day2_more = clutter.Label()
        day2_more.set_font_name(self.theme.getFont() + \
            str(self.ui.y(0.04167)) + "px")
        day2_more.set_color(self.theme.getColor("text"))
        day2_more.set_text("High: " + day["High"] + "   Low: " + \
            day["Low"] + "\nCondition: " + day["Condition"])
        day2_more.set_position(self.ui.x(0.13), self.ui.y(0.75))
        day2_more.show()
        self.group.add(day2_more)

        # Show day 3
        day = forecasts[2]

        # Weather Image
        weather_image = clutter.Texture()
        pix_buffer = gtk.gdk.pixbuf_new_from_file(day["Image"])
        weather_image.set_position(self.ui.x(0.40), self.ui.y(0.50))
        weather_image.set_pixbuf(pix_buffer)
        weather_image.show()
        self.group.add(weather_image)

        # Show day 3
        day3 = clutter.Label()
        day3.set_font_name(self.theme.getFont() + " bold " + \
            str(self.ui.y(0.04167)) + "px")
        day3.set_color(self.theme.getColor("text"))
        day3.set_text(day["Day"])
        day3.set_position(self.ui.x(0.40), self.ui.y(0.70))
        day3.show()
        self.group.add(day3)

        # Show day 3
        day3_more = clutter.Label()
        day3_more.set_font_name(self.theme.getFont() + \
            str(self.ui.y(0.04167)) + "px")
        day3_more.set_color(self.theme.getColor("text"))
        day3_more.set_text("High: " + day["High"] + "   Low: " + \
            day["Low"] + "\nCondition: " + day["Condition"])
        day3_more.set_position(self.ui.x(0.40), self.ui.y(0.75))
        day3_more.show()
        self.group.add(day3_more)

        # Show day 4
        day = forecasts[3]

        # Weather Image
        weather_image = clutter.Texture()
        pix_buffer = gtk.gdk.pixbuf_new_from_file(day["Image"])
        weather_image.set_position(self.ui.x(0.70), self.ui.y(0.50))
        weather_image.set_pixbuf(pix_buffer)
        weather_image.show()
        self.group.add(weather_image)

        # Show day 4
        day4 = clutter.Label()
        day4.set_font_name(self.theme.getFont() + " bold " + \
            str(self.ui.y(0.04167)) + "px")
        day4.set_color(self.theme.getColor("text"))
        day4.set_text(day["Day"])
        day4.set_position(self.ui.x(0.70), self.ui.y(0.70))
        day4.show()
        self.group.add(day4)

        # Show day 4
        day4_more = clutter.Label()
        day4_more.set_font_name(self.theme.getFont() + \
            str(self.ui.y(0.04167)) + "px")
        day4_more.set_color(self.theme.getColor("text"))
        day4_more.set_text("High: " + day["High"] + "   Low: " + \
            day["Low"] + "\nCondition: " + day["Condition"])
        day4_more.set_position(self.ui.x(0.70), self.ui.y(0.75))
        day4_more.show()
        self.group.add(day4_more)

        self.group.show()
        ui.getStage().add(self.group)

    def get_group(self):
        """
        Get all actors of the screen.
        @return clutter.Group
        """
        return self.group

    def get_type(self):
        """Return screen type."""
        return Screen.NORMAL

    def get_name(self):
        """Return screen name (human readble)"""
        return "WeatherScreen"
