'''NoEffectTransition - Simply change from screen 1 to screen 2'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

from frontend.gui.transitions.transition import Transition

class NoEffectTransition(Transition):
    """
    NoEffectTransition 
    
    This is the most simple transition. It simply changes view from screen 1
    to screen 2 without any special effects.
    """

    def __init__(self, stage):
        '''Create new NoEffectTransition

        stage - clutter stage
        '''
        Transition.__init__(self, stage)
    
    def forward_effect(self, from_screen, to_screen):   
        """
        Change screen without effects. 'from_screen' is None when app starts up.
        @param from_screen: Screen object (Currently displayed screen)
        @param to_screen: Screen object (Screen displayed after transition)
        """
        if from_screen is not None:
            from_screen.get_group().hide()
        to_screen.get_group().show()
                
    def backward_effect(self, from_screen, to_screen):
        """
        Change to the previous screen.
        @param from_screen: Screen object (Currently displayed screen)
        @param to_screen: Screen object (Screen displayed after transition)
        """
        if from_screen is not None:
            from_screen.get_group().hide()
            self.stage.remove(from_screen.get_group())
        to_screen.get_group().show()
        
