'''Implements a grid menu that contains images'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import clutter
from frontend.gui.widgets.selector import Selector
from frontend.gui.widgets.grid_menu import GridMenu

class ImageMenu(GridMenu):
    """
    ImageMenu widget

    This widget inherits genral GridMenu widget and implements simple
    imagemenu.
    """

    def __init__(self, theme):
        """Initialize widget"""
        GridMenu.__init__(self, self.HORIZONTAL)

        self.set_item_size(200, 150)
        self.set_row_count(3)

        c = clutter.Rectangle()
        c.set_color((255, 255, 255, 128))
        c.set_border_width(5)
        c.set_border_color((255, 255, 255, 255))
        self.set_cursor(c)

    def set_animate(self, boolean):
        """
        Enable/disable widget animation.
        @param boolean: True if widget should be animated, otherwise False
        """
        self.animate = boolean
        #self.cursor.animate(boolean)

    def _update_selected_item(self, new_location, old_location=None):
        """
        Overridden method! See GridMenu API reference.
        @param new_location: Logical location of current menuitem
        @param old_location: Logical location of last menuitem
        """
        if old_location is not None:
            self.items[old_location].set_active(False)
        self.items[new_location].set_active(True)

    def _animate_items_on_state_change(self):
        """
        This method is called when widget is set active or inactive. This
        method allows widget to react this action.
        """
        try:
            if self.active:
                self.cursor.show()
                self.items[self.logical_position].set_active(True)
                self.set_opacity(255)
            else:
                self.cursor.hide()
                self.items[self.logical_position].set_active(False)
                self.set_opacity(128)
        except:
            pass # No items in menu

    def _scale_menuitem(self, menuitem):
        """
        Scale menuitem to fit into one cell. Preserves actor's aspect ratio.
        @param menuitem: clutter.Actor
        """
#        w, h = menuitem.get_size()
#
#        if w == self.cell_width and h == self.cell_height:
#            return # Already correct size, no scale needed
#
#        x_ratio = self.cell_width / float(w)
#        y_ratio = self.cell_height / float(h)
#        if x_ratio > y_ratio:
#            menuitem.set_scale(self.cell_height /float(h),
#                               self.cell_height / float(h))
#        else:
#            menuitem.set_scale(self.cell_width / float(w),
#                               self.cell_width / float(w))
