'''Simple Menuitem'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import clutter

class MenuItem(clutter.Group):
    """
    Simple menuitem widget.

    This is a base class for all menuitem types. MenuItems are supposed to be
    used in Menus which inherit GridMenu class.
    """

    def __init__(self, width, height):
        """
        Initialize menuitem.
        @param width: Width of the menuitem
        @param height: Height of the menuitem
        """
        clutter.Group.__init__(self)
        self.active = False
        self.data = None

        rect = clutter.Rectangle()
        rect.set_size(width, height)
        rect.set_color(clutter.Color(0, 0, 0, 0))
        rect.show()
        self.add(rect)

    def set_active(self, boolean):
        """
        Set menuitem active / inactive.
        @param boolean: use True to set active and False to set inactive
        """
        self.active = boolean

    def is_active(self):
        """
        Get widget status.
        @return True if widget is active, otherwise False
        """
        return self.active

    def set_userdata(self, data):
        """
        Set user data for this menuitem.
        @param id: ID string
        """
        self.data = data

    def get_userdata(self):
        """
        Get user data
        @return: string
        """
        return self.data
