'''Tests frontend.medialibrary.music.Album'''

__licence__ = "GPLv2"
__copyright__ = "2008, Matt Layman"
__author__ = "Matt Layman <laymansterms.dev@gmail.com>"

import sys
import os

sys.path += [os.path.abspath('..')]

from tests.Music_test import TestMusic

from frontend.medialibrary.music import Album, AlbumHasNoTracks
from frontend.medialibrary.music import Track

class TestAlbum(TestMusic):
    '''Tests Album'''

    def setUp(self):
        TestMusic.setUp(self)
        self.album = Album('album1', self.cursor)

    def tearDown(self):
        TestMusic.tearDown(self)

    def testAlbumConstructor(self):
        """testAlbumConstructor - Ensures that an Album object is properly
        constructed"""
        self.assertTrue(isinstance(self.album, Album))

    def testAlbumConstructorNot(self):
        """testAlbumConstructorNot - Ensures that an AlbumHasNoTracks exception
        is raised when the created album doesn't exist in the cache"""
        self.assertRaises(AlbumHasNoTracks, Album, 'foo', self.cursor)

    def testAlbumStr(self):
        """testAlbumStr - Ensures that title is returned in string
        conversion"""
        result = str(self.album)
        self.assertEqual(result, 'album1')

    def testAlbumGetTitle(self):
        """testAlbumGetTitle - Ensures that the album title is returned"""
        result = self.album.get_title()
        self.assertEqual(result, 'album1')

    def testAlbumHasAlbumArt(self):
        """testAlbumHasAlbumArt - Ensures that album art exists for the file"""
        album_artist = "artist0 - album1"
        album_artist = album_artist.encode("base64")
        album_art = os.path.join(self.art_path, album_artist + ".jpg")
        file = open(album_art, "wb")
        file.close()
        self.assertTrue(self.album.has_album_art())
        if os.path.exists(album_art):
            os.remove(album_art)

    def testAlbumHasAlbumArtNot(self):
        """testAlbumHasAlbumArtNot - Ensures that missing album art is reported
        back"""
        otherAlbum = Album('album1', self.cursor)
        self.assertFalse(otherAlbum.has_album_art())

    def testAlbumGetAlbumArtUrl(self):
        """testAlbumGetAlbumArtUrl - Ensures that the path to the album's art
        is returned"""
        result = self.album.get_album_art_url()
        album_artist = "artist0 - album1"
        album_artist = album_artist.encode("base64")
        album_art = os.path.join(self.art_path, album_artist + ".jpg")
        self.assertEqual(result, album_art)

    def testAlbumGetAlbumArtUrlNot(self):
        #TODO: doc string for get_album_art_url mentions raising an exception.
        # I don't see it anywhere handled anywhere.
        pass

    def testAlbumGetTracks(self):
        """testAlbumGetTracks - Ensures that all tracks for an album are
        returned"""
        result = self.album.get_tracks()
        self.assertEqual(len(result), 4)
        for i in result:
            self.assertTrue(isinstance(i, Track))

    def testAlbumGetNumberOfTracks(self):
        """testAlbumGetNumberOfTracks - Ensures correct number of tracks from
        album is returned"""
        self.assertEqual(self.album.get_number_of_tracks(), 4)

    def testAlbumGetYear(self):
        """testAlbumGetYear - Ensures year of album is returned"""
        self.assertEqual(self.album.get_year(), 0)

    def testAlbumGetGenre(self):
        """testAlbumGetGenre - Ensures genre of album is returned"""
        self.assertEqual(self.album.get_genre(), 'genre0')

    def testAlbumGetArtist(self):
        """testAlbumGetArtist - Ensures artist of album is returned"""
        self.assertEqual(self.album.get_artist(), 'artist0')

    def testAlbumGetTotalLength(self):
        """testAlbumGetArtist - Ensures total length of album is returned"""
        self.assertEqual(self.album.get_total_length(), 8)

