'''Test feed configs'''
# pylint: disable-msg=C0301

__licence__ = "GPLv2"
__copyright__ = "2008, Joshua Scotton"
__author__ = "Joshua Scotton <josh@joshuascotton.com>"

import unittest
import sys
import os
import ConfigParser

from ConfigParser import ParsingError

try:
    import pygtk
    pygtk.require("2.4")
except:
    pass
try:
    import gtk
    import gtk.glade
except:
    sys.exit(1)

sys.path += [os.path.abspath('..')]

from utils.logger import Logger
from utils.feed_utils import FeedConfigTools

class FeedConfigToolsTest(unittest.TestCase):
    '''Test feed config tools'''

    def setUp(self, debug=False):
        """Sets up everything for the test"""

        self.debug = debug
        if(self.debug):
            self.logger = Logger('./tests.log').getLogger(
                'FeedConfigToolsTest')

        #initialise config file
        self.configPath = './data/FeedConfigTools/content.conf'
        self.config = ConfigParser.ConfigParser()
        try:
            self.config.readfp(open(os.path.expanduser(self.configPath)))
        except:
            print "Error with content.conf in FeedConfigToolsTest setUp()"
            sys.exit(1)

        #setup feeds and model
        self.feeds = self.config.get("RSS", "feeds").split(';')
        self.model = gtk.ListStore(str)
        for i in range(len(self.feeds)):
            self.model.insert(i, [self.feeds[i]])

        self.files = ["./data/FeedConfigTools/test.opml"]
        self.feedList = [
            "http://newsrss.bbc.co.uk/rss/newsonline_uk_edition/front_page/rss.xml",
            "http://www.hackaday.com/rss.xml",
            "http://edevelop.org/planet/rss20.xml",
            "http://feeds.feedburner.com/BlenderartMagazine"
            ]

    def tearDown(self):
        """restores config to how it should be to start off"""
        self.config.set("RSS", "feeds",
            "http://www.joshuascotton.com/main/feed")
        if(self.debug):
            self.logger.debug("About to refresh test config file")
        try:
            self.config.write(file(self.configPath, 'w'))
            if(self.debug):
                self.logger.debug("Refreshed test config file")
        except:
            if(self.debug):
                self.logger.debug("Can't refresh test config file")
            print "configuration file error, please take a look at: " + \
                self.configPath

    def testAddFileFeedsToWidget001(self):
        """tests if self.feeds is updated correctly"""
        FeedConfigTools().add_file_feeds_to_widget(
            self.files,
            self.config,
            self.model,
            self.feeds,
            self.configPath,
            False
            )
        if(self.debug):
            self.logger.debug("OUTPUT: self.feeds = " + str(self.feeds))
        self.assertEqual(self.feeds, [
            "http://www.joshuascotton.com/main/feed",
            "http://newsrss.bbc.co.uk/rss/newsonline_uk_edition/front_page/rss.xml",
            "http://www.hackaday.com/rss.xml",
            "http://edevelop.org/planet/rss20.xml",
            "http://feeds.feedburner.com/BlenderartMagazine"
            ])

    def testAddFileFeedsToWidget002(self):
        """tests if self.config is updated correctly"""
        FeedConfigTools().add_file_feeds_to_widget(
            self.files,
            self.config,
            self.model,
            self.feeds,
            self.configPath,
            False
            )
        if(self.debug):
            self.logger.debug(
                "OUTPUT: self.config.get(\"RSS\", \"feeds\").split(';') = " +
                str(self.config.get("RSS", "feeds").split(';') ))
        self.assertEqual(self.config.get("RSS", "feeds").split(';'), [
            "http://www.joshuascotton.com/main/feed",
            "http://newsrss.bbc.co.uk/rss/newsonline_uk_edition/front_page/rss.xml",
            "http://www.hackaday.com/rss.xml",
            "http://edevelop.org/planet/rss20.xml",
            "http://feeds.feedburner.com/BlenderartMagazine"
            ])

    def testAddFeedsToWidget001(self):
        """testAddFeedsToWidget001: tests if self.feeds is updated correctly"""
        FeedConfigTools().add_feeds_to_widget(
            self.feedList,
            self.config,
            self.model,
            self.feeds,
            self.configPath
            )
        if(self.debug):
            self.logger.debug("OUTPUT: self.feeds = " + str(self.feeds))
        self.assertEqual(self.feeds, [
            "http://www.joshuascotton.com/main/feed",
            "http://newsrss.bbc.co.uk/rss/newsonline_uk_edition/front_page/rss.xml",
            "http://www.hackaday.com/rss.xml",
            "http://edevelop.org/planet/rss20.xml",
            "http://feeds.feedburner.com/BlenderartMagazine"
            ])

    def testAddFeedsToWidget002(self):
        """testAddFeedsToWidget002: tests if self.config is updated correctly"""
        FeedConfigTools().add_feeds_to_widget(
            self.feedList,
            self.config,
            self.model,
            self.feeds,
            self.configPath
            )
        if(self.debug):
            self.logger.debug(
                "OUTPUT: self.config.get(\"RSS\", \"feeds\").split(';') = " +
                str(self.config.get("RSS", "feeds").split(';')))
        self.assertEqual(self.config.get("RSS", "feeds").split(';'), [
            "http://www.joshuascotton.com/main/feed",
            "http://newsrss.bbc.co.uk/rss/newsonline_uk_edition/front_page/rss.xml",
            "http://www.hackaday.com/rss.xml",
            "http://edevelop.org/planet/rss20.xml",
            "http://feeds.feedburner.com/BlenderartMagazine"
            ])

