'''Test feed entry parser'''
# pylint: disable-msg=C0301

__licence__ = "GPLv2"
__copyright__ = "2008, Joshua Scotton"
__author__ = "Joshua Scotton <josh@joshuascotton.com>"

import unittest
import sys
import os
import ConfigParser
import logging

from ConfigParser import ParsingError

try:
    import pygtk
    pygtk.require("2.4")
except:
    pass
try:
    import gtk
    import gtk.glade
except:
    sys.exit(1)

sys.path += [os.path.abspath('..')]

from utils.logger import Logger
from utils.feed_utils import FeedEntryParser

class FeedEntryParserTest(unittest.TestCase):
    '''Test the FeedEntryParser'''

    def setUp(self, debug=False):
        """Sets up everything for the test"""
        self.debug = debug
        if(self.debug):
            self.logger = Logger('./tests.log').getLogger(
                'FeedEntryParserTest')

        #test strings
        self.testString001 = "<body><b>Hello World</body></b>"
        self.testString002 = "<body><span id=\"001\"><u><b>&nbsp;He<i>l</i>lo <big>World</b></big><s>!</s><sub>!</sub><div><sup>!</sup><small>!</small><tt></div>...</tt></u></span></body>"
        self.resultString001 = "Hello World"
        self.resultString002 = "<span id=\"001\"><u><b>&nbsp;He<i>l</i>lo <big>World</b></big><s>!</s><sub>!</sub><sup>!</sup><small>!</small><tt>...</tt></u></span>"
        self.resultString003 = "<body>Hello World</body>"
        self.resultString004 = "<span id=\"001\"><u><b> He<i>l</i>lo <big>World</b></big><s>!</s><sub>!</sub><sup>!</sup><small>!</small><tt>...</tt></u></span>"

    def testStripTags001(self):
        """tests if FeedEntryParser strips all tags from input using default
        tags
        """
        output = FeedEntryParser().strip_tags(self.testString001)
        if(self.debug):
            self.logger.debug("EXPECTED: " + self.resultString001)
            self.logger.debug("OUTPUT: " + output)
        self.assertEqual(output, self.resultString001)

    def testStripTags002(self):
        """tests if FeedEntryParser strips all tags from input using
        non-default tags
        """
        output = FeedEntryParser().strip_tags(self.testString001, ["body"])
        if(self.debug):
            self.logger.debug("EXPECTED: " + self.resultString003)
            self.logger.debug("OUTPUT: " + output)
        self.assertEqual(output, self.resultString003)

    def testStripNonPangoTags(self):
        """testStripNonPangoTags: tests if FeedEntryParser strips all
        non-pango tags from input
        """
        output = FeedEntryParser().strip_non_pango_tags(self.testString002)
        if(self.debug):
            self.logger.debug("EXPECTED: " + self.resultString002)
            self.logger.debug("OUTPUT: " + output)
        self.assertEqual(output, self.resultString002)

    def testConvert(self):
        """tests if FeedEntryParser converts input to correct output"""
        output = FeedEntryParser().convert(self.testString002)
        if(self.debug):
            self.logger.debug("EXPECTED: " + self.resultString004)
            self.logger.debug("OUTPUT: " + output)
        self.assertEqual(output, self.resultString004)

