'''Tests LyricsDownloader'''

__licence__ = "GPLv2"
__copyright__ = "2008, Michael Charclo"
__author__ = "Michael Charclo <charclo.michael@gmail.com>"

import unittest
import sys
import os
import urllib

sys.path += [os.path.abspath('..')]

from utils.lyrics_downloader import LyricsDownloader

class testLyricsDownloader(unittest.TestCase):
    def setUp(self):
        pass

    def testLyricsConstructor(self):
        '''testMetadataConstructor - Ensures instantiation of Lyric Downloader
        class'''
        callback = False
        lyrics = LyricsDownloader('Foo', 'Bar', callback)
        self.assertTrue(isinstance(lyrics, LyricsDownloader))

    def testCleanUpArtistTitle(self):
        '''testCleanUpArtistTitle - Ensures the artist is converted for use in
        the url'''
        callback = False
        lyrics = LyricsDownloader('Foo', "Test 123 ()$^*=:;|#@}{][!,.-_\\&'" ,
                                   callback)
        lyrics._clean_up_artist_title()
        self.assertEqual(lyrics.artist,
                         'test%20123%20()$^*=:;|#@}{][!,.-_\\%%')

    def testGetLyricsXML(self):
        '''testGetLyricsXML - Ensures the lyrics are fetched correctly'''
        callback = False
        lyrics = LyricsDownloader('On A Plain', 'Nirvana', callback)
        lyrics._clean_up_artist_title()
        test_xml_path = urllib.urlopen(os.path.abspath('.') + \
                        '/data/LyricsDownloader/lyrics_xml')
        test_xml = test_xml_path.read()
        self.assertEqual(lyrics._get_lyrics_xml(), test_xml)

    def testParseLyricsXML(self):
        '''testParseLyricsXML - Ensures the lyrics are parsed correctly'''
        callback = False
        lyrics = LyricsDownloader('On A Plain', 'Nirvana', callback)
        test_xml_path = urllib.urlopen(os.path.abspath('.') + \
                        '/data/LyricsDownloader/lyrics_xml')
        test_xml = test_xml_path.read()
        final_lyrics = lyrics._parse_lyrics_xml(test_xml)
        temp = open(os.path.abspath('.') + \
                        '/data/LyricsDownloader/final_lyrics')
        final_lyrics_from_file = temp.read()
        final_lyrics_from_file = final_lyrics_from_file
        self.assertEqual(final_lyrics, final_lyrics_from_file)

if __name__ == "__main__":
    suite = unittest.TestLoader().loadTestsFromTestCase(testLyricsDownloader)
    result = unittest.TextTestRunner(descriptions=2, verbosity=2).run(suite)
