'''Test the thumbnailer abstract class'''

import unittest
import sys
import os

sys.path += [os.path.abspath('..')]

from utils import thumbnailer

class ThumbnailerTest(unittest.TestCase):
    '''Test the thumbnailer abstract class'''

    def setUp(self):
        '''See unittest.TestCase'''
        self.debug = False

    def testValidThumbnailType(self):
        '''Test creating thumbnails with a valid type'''
        thumbnailer_image = thumbnailer.Thumbnailer(
            os.path.abspath('.') + '/data/ImageThumbnailer/test.jpg', 'image')
        self.assertTrue(isinstance(thumbnailer_image, thumbnailer.Thumbnailer))

        thumbnailer_video = thumbnailer.Thumbnailer(
            os.path.abspath('.') + '/data/VideoThumbnailer/test.avi', 'video')
        self.assertTrue(isinstance(thumbnailer_video, thumbnailer.Thumbnailer))

        thumbnailer_recording = thumbnailer.Thumbnailer(
            os.path.abspath('.') + '/data/VideoThumbnailer/test.avi',
            'recording')
        self.assertTrue(isinstance(thumbnailer_recording,
            thumbnailer.Thumbnailer))

    def testInvalidThumbnailType(self):
        '''Test creating thumbnails with an invalid type'''
        self.assertRaises(thumbnailer.ThumbnailerException,
            thumbnailer.Thumbnailer,
            os.path.abspath('.') + '/data/ImageoThumbnailer/test.jpg',
            'alskdjfhg')

    def testValidFileToThumbnail(self):
        '''Test image file thumbnailing'''
        thumbnailer_image = thumbnailer.Thumbnailer(
            os.path.abspath('.') + '/data/ImageThumbnailer/test.jpg', 'image')
        self.assertTrue(isinstance(thumbnailer_image, thumbnailer.Thumbnailer))

    def testInvalidFileToThumbnail(self):
        '''Test thumbnailing of a non-existent file'''
        self.assertRaises(thumbnailer.ThumbnailerException,
            thumbnailer.Thumbnailer,
            os.path.abspath('.') + '/data/ImageThumbnailer/alskdjfhg.jpg',
            'image')

    def testAbstractCreateThumbnailer(self):
        '''Tests trying to create the abstract thumbnailer'''
        thumbnailer_test = thumbnailer.Thumbnailer(
            os.path.abspath('.') + '/data/ImageThumbnailer/test.jpg', 'image')
        self.assertRaises(Exception, thumbnailer_test.create_thumbnail)
