'''Tests VideoThumbnailer'''

import unittest
import sys
import os

sys.path += [os.path.abspath('..')]

from utils.video_thumbnailer import VideoThumbnailer, VideoThumbnailerException
from utils.thumbnailer import ThumbnailerException

class VideoThumbnailerTest(unittest.TestCase):
    '''Tests VideoThumbnailer'''

    def setUp(self):
        self.debug = False

    def testThumbnailerConstructor(self):
        '''Tests instantiation of thumbnailer class'''
        self.thumbnailer = VideoThumbnailer(
            os.path.abspath('.') +
            '/data/VideoThumbnailer/test.avi',
            src='video')
        self.assertTrue(isinstance(self.thumbnailer, VideoThumbnailer))

    def testThumbnailerConstructorFilenameIsFolder(self):
        '''Tests proper handling of folders instead of files'''
        self.assertRaises(ThumbnailerException,
            VideoThumbnailer, os.path.abspath('.'))

    def testThumbnailerConstructorFilenameExists(self):
        '''Tests existence of file to thumbnail'''
        self.assertRaises(ThumbnailerException,
            VideoThumbnailer, os.path.abspath('foo-bar-baz'))

    def testThumbnailerConstructorSrc(self):
        '''Tests valid source types'''
        self.assertRaises(ThumbnailerException,
            VideoThumbnailer, os.path.abspath('.') +
            '/data/VideoThumbnailer/test.avi', src="foo")

    def testThumbnailAvi(self):
        '''Tests thumbnailing of Avi file'''
        thumbnailer = VideoThumbnailer(
            os.path.abspath('.') + '/data/VideoThumbnailer/test.avi',
            src='video')
        if os.path.exists(thumbnailer._thumb_file):
            os.remove(thumbnailer._thumb_file)
        thumbnailer.create_thumbnail()
        if self.debug:
            print 'Expecting thumbnail : %s' % (self.thumbnailer._thumb_file)
        self.assertTrue(os.path.exists(thumbnailer._thumb_file))

    def testThumbnailFlv(self):
        '''Tests thumbnailing of a flash file'''
        thumbnailer = VideoThumbnailer(
            os.path.abspath('.') + '/data/VideoThumbnailer/test.avi',
            src='video')
        if os.path.exists(thumbnailer._thumb_file):
            os.remove(thumbnailer._thumb_file)
        thumbnailer.create_thumbnail()
        if self.debug:
            print 'Expecting thumbnail : %s' % (self.thumbnailer._thumb_file)
        self.assertTrue(os.path.exists(thumbnailer._thumb_file))

