'''Tests Weather'''

__licence__ = "GPLv2"
__copyright__ = "2008, Jamie Bennett"
__author__ = "Jamie Bennett <jamie@linuxuk.org>"

import unittest
import os
import sys

WEATHER_NS = 'http://xml.weather.yahoo.com/ns/rss/1.0'

sys.path += [os.path.abspath('..')]

from utils.weather import Weather

class WeatherTest(unittest.TestCase):
    """
    WeatherTest class
    Tests for the weather module.
    @author: Jamie Bennett <jamie@linuxuk.org>
    """
    def setUp(self):
        """Sets up everything for the test"""
        self.weather = Weather("Bath,England")

    def testWeatherFindRemote(self):
        """"Tests the weather code using a call to the outside world"""
        self.weather.set_location('Bath,England')
        self.weather.refresh()
        forecasts = self.weather.get_forecasts()
        try:
            today = forecasts[0]
            self.assertEqual(today["Day"],"Today")
        except:
            self.fail()

    def testWeatherSet(self):
        """Tests the setting of a forecast to a given state"""
        self.weather.clear_forecasts()
        forecast = {"Day":"Today", "Low":18, "High":20, "Condition":"Sunny"}
        self.weather.add_forecast(forecast)
        forecasts = self.weather.get_forecasts()
        today = forecasts[0]
        self.assertEqual(today["Day"],"Today")
        self.assertEqual(today["Low"], 18)
        self.assertEqual(today["High"], 20)
        self.assertEqual(today["Condition"],"Sunny")

    def testWeatherSetMetric(self):
        """Tests the metric setting of the weather module"""
        self.weather.set_metric (True)
        metric = self.weather.get_metric()
        self.assertEqual(metric, True)
        self.weather.set_metric (False)
        metric = self.weather.get_metric()
        self.assertEqual(metric, False)

    def testWeatherSetLocation(self):
        """Tests the location setting of the weather module"""
        self.weather.set_location('Bradford,England')
        self.assertEqual(self.weather.get_location(), 'Bradford,England')

