'''Generates thumbnails from image files'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"
__maintainer__ = "Paul Hummer <paul@ironlionsoftware.no.spam.com>"

import os
import Image
from utils import thumbnailer

class ImageThumbnailerException(Exception):
    '''An exception specific to the ImageThumbnailer'''

class ImageThumbnailer(thumbnailer.Thumbnailer):
    """Thumbnailer for image files."""

    def __init__(self, filename):
        """Create a new Image thumbnailer"""
        thumbnailer.Thumbnailer.__init__(self, filename, 'image')
        try:
            self.im = Image.open(self.filename)
        except:
            raise ImageThumbnailerException(
                'Error while opening file : %s' % self.filename)


    def create_thumbnail(self):
        """
        Method creates a new thumbnail and saves it to the Entertainer's
        thumbnail directory in PNG format. Thumbnail filename is a MD5
        hash of the given filename.
        """

        # Calculate new size here
        original_width = self.im.size[0]
        original_height = self.im.size[1]
        if original_width <= self.MAX_SIZE and (
            original_height <= self.MAX_SIZE):
            try:
                self.im.save(self._thumb_file, "JPEG",
                    quality=self.THUMB_QUALITY)
            except:
                raise ImageThumbnailerException('Error saving thumbnail')

        else:
            if original_width > original_height:
                width = self.MAX_SIZE
                height = (width * original_height) / original_width
            else:
                height = self.MAX_SIZE
                width = (height * original_width) / original_height
            try:
                self.im.thumbnail((width, height), Image.ANTIALIAS)
                self.im.save(self._thumb_file, "JPEG",
                    quality=self.THUMB_QUALITY)
            except:
                raise ImageThumbnailerException('Error saving thumbnail')

