'''Logger class'''

__license__ = 'GPLv2'
__copyright__ = '2008, Matt Layman'
__author__ = 'Matt Layman <laymansterms.dev@gmail.com>'

import os
import logging

from utils.configuration import Configuration

class Logger:
    '''Logger to record behind the scenes information.

    Tailored to the liking of the Entertainer developers.'''

    def __init__(self, filename=None):
        '''Create a new Entertainer logger object

        This logger creates the necessary customization for Entertainer
        logging and should be followed by a getLogger call.

        Example call:
        self.logger = Logger().getLogger(u'my.source.class')
        '''

        self._filename = filename

        if filename is None:
            self._cfg_path = Configuration().get_cfg_path()
            self._filename = os.path.expanduser(self._cfg_path +
                                u'entertainer.log')
            
        logging.basicConfig(
                    level=logging.DEBUG,
                    format='%(asctime)s %(name)s %(levelname)s %(message)s',
                    filename=self._filename,
                    filemode='a')

    def getLogger(self, logname=u''):
        '''Return a lower level logging object

        Since the low level log object does all the heavy lifting, it needs to
        be called.
        '''
        return logging.getLogger(logname)


